# prc01u.py (encoding: utf-8)
  # 身長の平均値が男女間で有意に異なるか t検定で検証
  # 結果をhtmlとして書き出す
import pandas as pd
from rpy2.robjects import r
from textwrap import dedent
import markdown as md
import pyrcmd as prc
prc.set_charset('utf-8')

src_file = 'data.csv'
rscript = dedent(u'''\
    dtf <- read.csv("%s", header=T, fileEncoding="%s")
    male = subset(dtf, 性別 == "男性")
    female = subset(dtf, 性別 == "女性")
    flag <- FALSE  # 不等分散を仮定
    t_test <- t.test(male$身長, female$身長, var.equal=flag)
    t_test
    ''') % (src_file, prc.get_charset(src_file))
md_tpl = dedent(u'''\
    # 身長の平均値に関する検証\n
    　身長の平均値が男女間で異なるかを見るため t検定(両側検定)を行った。\n
    　主な数値は次のとおり。\n
    - 統計量 t値: {t}
    - 自由度(df): {df}
    - 有意確率(p値): {p_value}
    - 平均値: 男性 {mean_of_x:.1f}, 女性 {mean_of_y:.1f}
    - 95％信頼区間: {conf_int[0]:.4f} -- {conf_int[1]:.4f}\n
    　有意確率が低いことから(p<0.05)、平均値が等しくないと判断される。\n
    　なお、統計解析ソフトRの出力結果は下のとおり。\n
    ~~~~
    {sink}
    ~~~~\n
    ----\n\n
    ''')

sink_data = prc.rexec(rscript)  # Rコマンドの実行
t_test = r['t_test']  # rp2オブジェクト(ListVector)として取得
pod = prc.rp2pd(t_test)  # ListVectorを変換(OrderedDictになる)
fod = prc.format_od(pod)  # テンプレート用に変換
fod['sink'] = sink_data  # 統計Rの出力を組み入れる
md_src = md_tpl.format(**fod)  # markdown原稿の生成
title = u't検定'  # htmlのタイトル
body = md.markdown(md_src, ["fenced_code"])
html = prc.html_template % ('utf-8', title, body)
prc.write_text("prc01u.htm", html, 'utf-8')
