# prc03u.py (encoding: utf-8)
  # jinja2の条件分岐の例。htmlを生成
  # 身長と体重の分布状況を表示。summary, hist, shapiro.test
import pandas as pd
from rpy2.robjects import r
from jinja2 import Template
from textwrap import dedent
import markdown as md
import pyrcmd as prc
prc.set_charset('utf-8')

src_file = 'data.csv'
rscript = dedent(u'''\
    dtf <- read.csv("%s", header=T, na.string="", fileEncoding="%s")
    jp_names <- c("身長", "体重")
    eng_names <- c("height", "weight")
    xx = list()  # 統計処理結果を入れるための空のリスト
    for (i in 1:length(jp_names)) {
        jp <- jp_names[i]
        eng <- eng_names[i]
        vct <- dtf[[jp]]  # 着目の列をvctに
        vct <- vct[!is.na(vct)]  # 欠損値の除去
        smr <- summary(vct)  # 分布の要約を得る
        shp <- shapiro.test(x=vct)  # 正規性の検証
        xx[[eng]] <- list(summary=smr, shapiro=shp)
        xx[[eng]]$name <- jp
        fname <- sprintf("prc03u_%%s.png", eng)
        png(fname, width=240, height=240)
        hist(vct, main=eng)
        dev.off()  # 描画デバイスを閉じる
        xx[[eng]]$png_name <- fname
    }
    ''') % (src_file, prc.get_charset(src_file))

md_tpl = Template(dedent(u'''\
    # {{name}}の分布\n
    　{{name}}の分布状況(summary)は下のとおり。\n
    {{summary}}\n
    　ヒストグラムを描くと下のようになった。\n
    ![{{name}}のヒストグラム]({{png_name}})\n
    　正規性を検証するため shapiro.test を行ったところ、
    次のような結果だった。\n
    - 統計量 W: {{shapiro.W}}
    - 有意確率 p: {{shapiro.p_value}}\n
    {% if shapiro.p_value < 0.05 %}
    {{name}}については、p<0.05 であることから
    「正規分布に即している」(帰無仮説)が棄却される。\n
    つまり、正規分布ではないと判断できる。
    {% else %}
    {{name}}については、p>=0.05 であることから
    「正規分布に即している」(帰無仮説)を棄却できない。\n
    つまり、正規分布に即しているとみることができる。
    {% endif %}
    --------\n\n
    '''))

prc.rexec(rscript, sink=False)  # Rコマンドの実行
xx = r['xx']  # rpy2のオブジェクトを得る
pod = prc.rp2pd(xx)  # pandas型などに変換
for key in ['height', 'weight']:
    ser = pod[key]['summary']  # summaryの結果(Series)
    ser = ser.round(2)  # 数値のまるめ処理
    df = pd.concat([ser], axis=1).T  # DataFrameに変換
    pod[key]['summary'] = df
fod = prc.format_od(pod, index=False)  # テンプレートで扱えるように変換

md_src = ''
for key in ['height', 'weight']:
    md_src = md_src + md_tpl.render(fod[key])
title = u'分布状況の確認と正規性の検証'  # htmlのタイトル
body = md.markdown(md_src, extensions=["tables"])
html = prc.html_template % ('utf-8', title, body)
prc.write_text("prc03u.htm", html, 'utf-8')
